<?php

namespace Drupal\simple_sitemap_pagination_url\PathProcessor;

use Drupal\Core\PathProcessor\InboundPathProcessorInterface;
use Drupal\Core\PathProcessor\OutboundPathProcessorInterface;
use Drupal\Core\Render\BubbleableMetadata;
use Symfony\Component\HttpFoundation\Request;

/**
 * Processes sitemap paths.
 */
class SitemapPathProcessor implements InboundPathProcessorInterface, OutboundPathProcessorInterface {

  /**
   * {@inheritdoc}
   */
  public function processInbound($path, Request $request) {
    // Convert /sitemap-N.xml to /sitemap/page/N.
    if (preg_match('/^\/sitemap-(\d+)\.xml$/', $path, $matches)) {
      $page = $matches[1];
      return "/sitemap/page/{$page}";
    }

    // Convert /sitemap.xml to /sitemap/page/0.
    if ($path === '/sitemap.xml') {
      return '/sitemap/page/0';
    }

    // Convert /sitemaps/{variant}/sitemap-N.xml to /sitemaps/{variant}/page/N.
    if (preg_match('#^/sitemaps/([^/]+)/sitemap-(\d+)\.xml$#', $path, $matches)) {
      $variant = $matches[1];
      $page = $matches[2];
      return "/sitemaps/$variant/page/$page";
    }

    // Convert /sitemaps/{variant}/sitemap.xml to /sitemaps/{variant}/page/0.
    if (preg_match('#^/sitemaps/([^/]+)/sitemap\.xml$#', $path, $matches)) {
      $variant = $matches[1];
      return "/sitemaps/$variant/page/0";
    }

    return $path;
  }

  /**
   * {@inheritdoc}
   */
  public function processOutbound($path, &$options = [], ?Request $request = NULL, ?BubbleableMetadata $bubbleable_metadata = NULL) {
    // Convert /sitemap/page/N back to /sitemap-N.xml.
    if (preg_match('/^\/sitemap\/page\/(\d+)$/', $path, $matches)) {
      $page = (int) $matches[1];
      if ($page === 0) {
        return '/sitemap.xml';
      }
      return "/sitemap-{$page}.xml";
    }

    // Convert /sitemaps/{variant}/page/N back
    // to /sitemaps/{variant}/sitemap-N.xml.
    if (preg_match('#^/sitemaps/([^/]+)/page/(\d+)$#', $path, $matches)) {
      $variant = $matches[1];
      $page = (int) $matches[2];
      return $page == 0
        ? "/sitemaps/$variant/sitemap.xml"
        : "/sitemaps/$variant/sitemap-$page.xml";
    }

    return $path;
  }

}
